/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "function-factory.h"
#include "help-function.h"
#include "getview-function.h"
#include "setview-function.h"
#include "checkout-function.h"
#include "checkin-function.h"
#include "getlayer-function.h"
#include "getversion-function.h"
#include "cvslog-function.h"
#include "add-function.h"



function_factory::function_factory ()
{
  functions.insert (ValuePair ("help", HELP));
  functions.insert (ValuePair ("getview", GETVIEW));
  functions.insert (ValuePair ("setview", SETVIEW));
  functions.insert (ValuePair ("checkout", CHECKOUT));
  functions.insert (ValuePair ("getlayer", GETLAYER));
  functions.insert (ValuePair ("getversion", GETVERSION));
  functions.insert (ValuePair ("cvslog", CVSLOG));
  functions.insert (ValuePair ("checkin", CHECKIN));
  functions.insert (ValuePair ("add", ADD));
}



function_factory::~function_factory ()
{
};



ctl_function *function_factory::generate (const std::string & key)
{
  MapType::iterator iter;

  iter = functions.find (key);
  if (iter == functions.end ())
    return 0;
    
  switch ((*iter).second)
  {
    case HELP:
      return new help_function ();

    case GETVIEW:
      return new getview_function ();

    case SETVIEW:
      return new setview_function ();

    case CHECKOUT:
      return new checkout_function ();

    case GETLAYER:
      return new getlayer_function ();

    case GETVERSION:
      return new getversion_function ();

    case CVSLOG:
      return new cvslog_function ();

    case CHECKIN:
      return new checkin_function ();

    case ADD:
      return new add_function ();
  }

  return 0;
}
