/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "base-socket.h"

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "connected-socket.h"



base_socket::~base_socket()
{
  close ();
}



bool base_socket::create()
{
  sockfd = ::socket (AF_INET, SOCK_STREAM, 0);
  if (sockfd == -1)
    return false;

  return true;
}



bool base_socket::create (int port)
{
  if (!create ())
    return false;
   
  address.sin_family = AF_INET;
  address.sin_port = htons (port);
  address.sin_addr.s_addr = INADDR_ANY;
  ::bzero (&(address.sin_zero), 8);
   
  if (::bind (sockfd, (struct sockaddr *) &address, sizeof (struct sockaddr)) == -1)
    return false;

  return true;
}



connected_socket *base_socket::connect (int port, const char *hostName)
{
  hostent *host;
  connected_socket *conn;
	
  if ((host = ::gethostbyname (hostName)) == NULL)
    return 0;

  if (sockfd == 0)
  {
    if (!create ())
      return 0;
  }

  address.sin_family = AF_INET;
  address.sin_port = htons (port);
  address.sin_addr = *((struct in_addr *) host->h_addr);
  ::bzero (&(address.sin_zero), 8);

  if (::connect (sockfd, (struct sockaddr *) &address, sizeof (struct sockaddr)) == -1)
    return 0;

  conn = new connected_socket (sockfd, address);

  create ();

  return conn;
}



bool base_socket::close ()
{
  int result;

  if (sockfd != 0)
  {
    result = ::close (sockfd);
    if (result == -1)
      return false;
  }

  return true;
}



bool base_socket::listen ()
{
  if (::listen (sockfd, 5) == -1)
    return false;
   
  return true;
}



connected_socket *base_socket::accept ()
{
  int newfd;
  struct sockaddr_in remote;
  socklen_t sin_size = sizeof (struct sockaddr_in);
  
  //sockaddr_in RemoteAddress;
  if ((newfd = ::accept (sockfd, (struct sockaddr *) &remote, &sin_size)) == -1)
    return 0;
  
  return new connected_socket (newfd, remote);
}
