/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "cvs-command.h"
#include <ext/stdio_filebuf.h>
#include "cvs-session.h"
#include "system-log.h"



cvs_command::cvs_command ()
{
}



cvs_command::~cvs_command ()
{
}



bool cvs_command::process_response (cvs_session &session)
{
  std::string response;

  // now analyze the response we get from the CVS server
  if (!session.read_line (&response))
    return false;
    
  while (response != "ok")
  {
    if (response == "error")
      return false;

    if (session.get_log () && (response.length () > 1))
    {
      if (((response[0] == 'M') || (response[0] == 'E')) && (response[1] == ' '))
      {
        *(session.get_log ()) << response << std::endl;
      }
    }

    if (!process_line (session, response))
      return false;

    if (!session.read_line (&response))
      return false;
  }

  return true;
}



bool cvs_command::establish_logging (cvs_session &session)
{
  system_log *log = system_log::instance ();
  char buffer[18];
  
  strcpy (buffer, "/tmp/cvsfs.XXXXXX");
  
  int handle = mkstemp (buffer);

  if (handle == -1)
    return false;

  log->debug << "cvs_command::establish_logging: Logfile allocated: " << buffer << std::endl;

  filename = buffer;

// workaround: gcc 3.3 --> 3.4 removed 3rd parameter of stdio_filebuf.
#ifdef _EXT_STDIO_FILEBUF
  __gnu_cxx::stdio_filebuf<char> *filebuf = new __gnu_cxx::stdio_filebuf<char> (handle, std::ios::out | std::ios::trunc, true, BUFSIZ);
#else
  __gnu_cxx::stdio_filebuf<char> *filebuf = new __gnu_cxx::stdio_filebuf<char> (handle, std::ios::out | std::ios::trunc, BUFSIZ);
#endif

  session.set_log (new std::ostream (filebuf));

  return true;
}
