/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "cvs-pserver-connection.h"
#include <sstream>
#include "base-socket.h"
#include "connected-socket.h"
#include "cvs-pserver-session.h"
#include "cvs-command.h"



const char *beginAuthRequest	= "BEGIN AUTH REQUEST";
const char *beginVerifyRequest	= "BEGIN VERIFICATION REQUEST";
const char *endAuthRequest	= "END AUTH REQUEST";
const char *endVerifyRequest	= "END VERIFICATION REQUEST";
const char *validResponses      = "Valid-responses ok error Valid-requests Checked-in New-entry Checksum Copy-file Updated Created Update-existing Merged Patched Rcs-diff Mode Mod-time Removed Remove-entry Set-static-directory Clear-static-directory Set-sticky Clear-sticky Template  Set-checkin-prog Set-update-prog Notified Module-expansion Wrapper-rcsOption M  Mbinary E F MT";

const char *successReply	= "I LOVE YOU";
const char *failedReply		= "I HATE YOU";



// password encoding table
static unsigned char shifts[] = {
    0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
   16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
  114,120, 53, 79, 96,109, 72,108, 70, 64, 76, 67,116, 74, 68, 87,
  111, 52, 75,119, 49, 34, 82, 81, 95, 65,112, 86,118,110,122,105,
   41, 57, 83, 43, 46,102, 40, 89, 38,103, 45, 50, 42,123, 91, 35,
  125, 55, 54, 66,124,126, 59, 47, 92, 71,115, 78, 88,107,106, 56,
   36,121,117,104,101,100, 69, 73, 99, 63, 94, 93, 39, 37, 61, 48,
   58,113, 32, 90, 44, 98, 60, 51, 33, 97, 62, 77, 84, 80, 85,223,
  225,216,187,166,229,189,222,188,141,249,148,200,184,136,248,190,
  199,170,181,204,138,232,218,183,255,234,220,247,213,203,226,193,
  174,172,228,252,217,201,131,230,197,211,145,238,161,179,160,212,
  207,221,254,173,202,146,224,151,140,196,205,130,135,133,143,246,
  192,159,244,239,185,168,215,144,139,165,180,157,147,186,214,176,
  227,231,219,169,175,156,206,198,129,164,150,210,154,177,134,127,
  182,128,158,208,162,132,167,209,149,241,153,251,237,236,171,195,
  243,233,253,240,194,250,191,155,142,137,245,235,163,242,178,152
};



cvs_pserver_connection::cvs_pserver_connection (const std::string &userid, const std::string &password,
						const std::string &server, const std::string &port,
						const std::string &root, const std::string &project)
: cvs_connection (server, 2401, root, project), cvs_userid (userid), cvs_password ("A")
{
  if (cvs_userid.length () == 0)
    cvs_userid = "anonymous";

  // encrypt password
  std::string::const_iterator iter;

  for (iter = password.begin (); iter != password.end (); ++iter)
    cvs_password += shifts[(unsigned char) (*iter)];

  if ((std::istringstream (port) >> cvs_port).fail ())
    cvs_port = 2401;
}



cvs_pserver_connection::cvs_pserver_connection (const cvs_pserver_connection &clone)
: cvs_connection (clone), cvs_userid (clone.cvs_userid), cvs_password (clone.cvs_password)
{
}



cvs_pserver_connection::~cvs_pserver_connection ()
{
}



cvs_connection *cvs_pserver_connection::clone ()
{
  return new cvs_pserver_connection (*this);
}



bool cvs_pserver_connection::test ()
{
  cvs_pserver_session *session = establish_connection ();
  if (!session)
    return false;

  bool result = do_test (session);

  close_connection (session);

  return result;
}



bool cvs_pserver_connection::login (cvs_pserver_session *& session)
{
  session = establish_connection ();
  if (!session)
    return false;

  if (do_login (session))
    return true;
  
  // something failed
  close_connection (session);
  
  return false;
}



cvs_pserver_session *cvs_pserver_connection::establish_connection ()
{
  base_socket base;
  connected_socket *sock = base.connect (2401, cvs_server.c_str ());
  if (!sock)
    return 0;

  return new cvs_pserver_session (*this, sock);
}



void cvs_pserver_connection::close_connection (cvs_pserver_session *&session)
{
  delete session;
  session = 0;
}



bool cvs_pserver_connection::do_test (cvs_pserver_session *session)
{
  session->send () << beginVerifyRequest << std::endl
                   << cvs_root << std::endl
		   << cvs_userid << std::endl
		   << cvs_password << std::endl
		   << endVerifyRequest << std::endl;

  std::string response;

  if ((session->read_line (&response)) && (response == successReply))
    return true;

  return false;
}



bool cvs_pserver_connection::do_login (cvs_pserver_session *session)
{
  session->send () << beginAuthRequest << std::endl
                   << cvs_root << std::endl
		   << cvs_userid << std::endl
		   << cvs_password << std::endl
		   << endAuthRequest << std::endl;

  std::string response;

  if ((session->read_line (&response)) && (response == successReply))
  {
    session->send () << "Root " << cvs_root << std::endl
                     << validResponses << std::endl
		     << "UseUnchanged" << std::endl;
    return true;
  }

  return false;
}



bool cvs_pserver_connection::execute_command (cvs_command &cmd)
{
  cvs_pserver_session *session;

  if (!login (session))
    return false;

  bool result = cmd.run (*session);

  close_connection (session);

  return result;
}
