/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __CVS_REPOSITORY_H__
#define __CVS_REPOSITORY_H__

#include "repository.h"
#include <time.h>
#include <string>
#include "directory.h"
#include "cvs-directory-entry.h"
#include "cvs-loadtree-thread.h"

// forward references
class cvs_connection;
class fs_layer_cvs;
class fs_layer_checkout;



class cvs_repository : public repository
{
  public:
    cvs_repository (const std::string &, cvs_connection *, const std::string &, const std::string &);
    ~cvs_repository ();

    virtual void startup ();
    virtual void shutdown ();

    fs_layer_cvs *create_cvs_layer ();
    fs_layer_checkout *create_checkout_layer ();

    void decrement_refcount ();

    directory *readdir (const std::string &);
    int getattr (const std::string &, struct stat *);
    bool get_fullname (const std::string &, std::string *);
    int checkout_file (const std::string &, char *, size_t);
    int checkin_file (const std::string &, char *, size_t);
    int get_log (const std::string &, char *, size_t);
    virtual int add_item (const std::string &, char *, size_t, fs_layer *);

  private:
//    void check_directory_tree ();
    bool check_entry_attributes (const std::string &, cvs_directory_entry &);
    bool create_directory (const std::string &, mode_t);
    int handle_response (const std::string &, char *, size_t);

    std::string storage_path;
    std::string cvs_storage_path;
    std::string checkout_storage_path;
    cvs_connection *connection;
    std::string repository;
    std::string label;
    cvs_directory_entry directory_tree;
    cvs_loadtree_thread tree_updater;
    int refcount;
};



#endif
