/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mutex.h"
#include "system-log.h"



mutex::mutex ()
{
  int ret = ::pthread_mutex_init (&fmutex, 0);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not initialize mutex. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }
}



mutex::~mutex ()
{
  ::pthread_mutex_destroy (&fmutex);
}



bool mutex::lock ()
{
  int ret = ::pthread_mutex_lock (&fmutex);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not lock mutex. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool mutex::try_lock ()
{
  int ret = ::pthread_mutex_trylock (&fmutex);
  
  return ret == 0;
}



bool mutex::unlock ()
{
  int ret = ::pthread_mutex_unlock (&fmutex);

  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not unlock mutex. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}
