/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "sema.h"
#include <errno.h>
#include "system-log.h"



semaphore::semaphore (int value)
{
  int ret = ::sem_init (&sem, 0, value);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not init semaphore. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }
}



semaphore::~semaphore ()
{
  ::sem_destroy (&sem);
}



bool semaphore::wait ()
{
  int ret = ::sem_wait (&sem);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not wait for semaphore. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool semaphore::try_wait ()
{
  int ret = ::sem_trywait (&sem);
  
  if ((ret != EAGAIN)  && (ret != 0))
  {
    system_log *log = system_log::instance ();

    log->error << "Can not try-wait for semaphore. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool semaphore::post ()
{
  int ret = ::sem_post (&sem);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not post to semaphore. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



int semaphore::value ()
{
  int value;
  int ret = ::sem_getvalue (&sem, &value);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not get value of semaphore. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return value;
}
