/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "system-log.h"

#include <syslog.h>
#include "syslog-file.h"



// mapping of the enum LogFacility to the values required by syslogd
const int FacilityMapping[] = { LOG_KERN,	// Kernel messages
				LOG_USER,	// user-level messages
				LOG_MAIL,	// mail system
				LOG_DAEMON,	// system daemons
				LOG_AUTH,	// authorization messages
				LOG_SYSLOG,	// syslogd internal messages
				LOG_LPR,	// line printer subsystem
				LOG_NEWS,	// network news subsystem
				LOG_UUCP,	// UUCP subsystem
				LOG_CRON,	// clock daemon
				LOG_AUTHPRIV,	// auth messages (private)
				LOG_FTP};	// ftp daemon



system_log *system_log::fInstance	= 0;
syslog_file *system_log::fFile		= 0;



system_log::system_log ()
: emergency (*fFile, syslog_stream::Emergency),
  alert (*fFile, syslog_stream::Alert),
  critical (*fFile, syslog_stream::Critical),
  error (*fFile, syslog_stream::Error),
  warning (*fFile, syslog_stream::Warning),
  notice (*fFile, syslog_stream::Notice),
  info (*fFile, syslog_stream::Info),
  debug (*fFile, syslog_stream::Debug)
{
}



system_log::~system_log ()
{
  delete fInstance;
  delete fFile;
}



system_log *system_log::instance ()
{
  if (fInstance == 0)
  {
    fFile = new syslog_file ("dummy", FacilityMapping[User]);
    fInstance = new system_log ();
  }

  return fInstance;
}



system_log *system_log::instance (const std::string & id, LogFacility logFacility)
{
  if (fInstance == 0)
  {
    fFile = new syslog_file (id, FacilityMapping[logFacility]);
    fInstance = new system_log ();
  }
  else
    fFile->reopen (id, FacilityMapping[logFacility]);

  return fInstance;
}
