/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "thread.h"
#include "system-log.h"



thread::thread ()
: running (false)
{
  ::pthread_attr_init (&pthread_attr);
}



thread::~thread ()
{
  ::pthread_attr_destroy (&pthread_attr);
}



bool thread::create ()
{
  int ret = ::pthread_create (&pthread, &pthread_attr, thread::start, this);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not start thread. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool thread::join ()
{
  int ret = ::pthread_join (pthread, 0);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not join to thread. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool thread::set_joinable ()
{
  int ret = ::pthread_attr_setdetachstate (&pthread_attr, PTHREAD_CREATE_JOINABLE);

  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not set thread attribute to joinable. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool thread::set_detached ()
{
  int ret = ::pthread_attr_setdetachstate (&pthread_attr, PTHREAD_CREATE_DETACHED);

  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not set thread attribute to joinable. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



void *thread::start (void *arg)
{
  static_cast<thread *> (arg)->running = true;
  static_cast<thread *> (arg)->run ();
  static_cast<thread *> (arg)->running = false;

  return 0;
}
